/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import java.util.HashSet;
import java.util.Objects;
import java.util.Set;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import com.fasterxml.jackson.annotation.JsonProperty;

import de.ozgcloud.antragsraum.common.VirusScanException;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.Setter;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
@RequiredArgsConstructor
class VirusScannerClient {

	private static final String FILE_REQUEST_KEY = "FILES";

	@Value("${clamav.scanUrl}")
	private String clamAVScanUrl;

	private final RestTemplate restTemplate;

	Set<String> scan(MultipartFile file) {
		Set<String> viruses = new HashSet<>();

		if (Objects.nonNull(file)) {
			LOG.debug("Scanning file {} for viruses using ClamAV at {}", file.getOriginalFilename(), clamAVScanUrl);

			MultiValueMap<String, Object> requestBody = new LinkedMultiValueMap<>();
			requestBody.add(FILE_REQUEST_KEY, file.getResource());
			HttpEntity<MultiValueMap<String, Object>> request = new HttpEntity<>(requestBody);

			var response = restTemplate.postForEntity(clamAVScanUrl, request, VirusScanResponse.class);

			if (response.getStatusCode().isError()) {
				LOG.error("Error scanning file for virus. ClamAV Service returned code {}", response.getStatusCode());
				throw new VirusScanException(file.getName());
			}

			var responseBody = Objects.requireNonNull(response.getBody());
			if (!responseBody.isSuccess()) {
				LOG.error("Scanning file for virus was not successfully. ClamAV Response {}", responseBody);
				throw new VirusScanException(file.getName());
			}

			var result = responseBody.getData().getResult()[0];
			viruses.addAll(Set.of(result.getViruses()));
		} else {
			LOG.warn("Tried to scan null file, will return empty ScanResult Set");
		}

		return viruses;
	}

	@Getter
	@Setter
	static class VirusScanResponse {

		private VirusScanData data;
		private boolean success;

	}

	@Getter
	@Setter
	static class VirusScanData {

		private VirusScanResult[] result;

	}

	@Getter
	@Setter
	static class VirusScanResult {

		private String name;
		@JsonProperty("is_infected")
		private boolean infected;
		private String[] viruses;

	}

}
