/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.attachments.OzgFileTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.util.unit.DataSize;

@ExtendWith(MockitoExtension.class)
class FileValidatorTest {

	static final MockMultipartFile TEST_FILE = new MockMultipartFile("file", FILE_NAME, CONTENT_TYPE, DATA);
	static final MockMultipartFile TEST_INVALID_FILE = new MockMultipartFile("file", FILE_NAME, INVALID_CONTENT_TYPE, DATA);
	static final MockMultipartFile TEST_TOO_LARGE_FILE = new MockMultipartFile("file", FILE_NAME, CONTENT_TYPE, new byte[150_000_000]);

	@InjectMocks
	private FileValidator validator;

	@Mock
	private FileProperties fileProperties;

	@Nested
	class TestFileSizeIsValid {

		@BeforeEach
		void init() {
			when(fileProperties.getMaxFileSize()).thenReturn(DataSize.ofMegabytes(100));
		}

		@Test
		void shouldBeValid() {
			var result = validator.fileSizeIsValid(99_000_000);

			assertThat(result).isTrue();
		}

		@Test
		void shouldBeInvalid() {
			var result = validator.fileSizeIsValid(150_000_000);

			assertThat(result).isFalse();
		}

	}

	@Nested
	class TestFileContentTypeIsValid {

		@Test
		void shouldBeValid() {
			when(fileProperties.getBlockedContentTypes()).thenReturn(new String[] { INVALID_CONTENT_TYPE });

			var result = validator.fileContentTypeIsValid(CONTENT_TYPE);

			assertThat(result).isTrue();
		}

		@Test
		void shouldBeInvalid() {
			when(fileProperties.getBlockedContentTypes()).thenReturn(new String[] { INVALID_CONTENT_TYPE });

			var result = validator.fileContentTypeIsValid(INVALID_CONTENT_TYPE);

			assertThat(result).isFalse();
		}

		@Test
		void shouldBeInvalidBecauseNotSet() {
			var result = validator.fileContentTypeIsValid(null);

			assertThat(result).isFalse();
		}

	}

	@Nested
	class TestIsValid {

		@BeforeEach
		void init() {
			when(fileProperties.getMaxFileSize()).thenReturn(DataSize.ofMegabytes(100));
		}

		@Test
		void shouldHaveInvalidFileSize() {
			var result = validator.isValid(TEST_TOO_LARGE_FILE);

			assertThat(result).isFalse();
		}

		@Test
		void shouldHaveInvalidContentType() {
			when(fileProperties.getBlockedContentTypes()).thenReturn(new String[] { INVALID_CONTENT_TYPE });

			var result = validator.isValid(TEST_INVALID_FILE);

			assertThat(result).isFalse();
		}

		@Test
		void shouldBeValid() {
			when(fileProperties.getBlockedContentTypes()).thenReturn(new String[] { INVALID_CONTENT_TYPE });

			var result = validator.isValid(TEST_FILE);

			assertThat(result).isTrue();
		}

	}

}