/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import de.ozgcloud.antragsraum.attachments.FileIdentificationData;
import de.ozgcloud.antragsraum.attachments.FileService;
import de.ozgcloud.antragsraum.attachments.OzgFile;
import de.ozgcloud.antragsraum.attachments.OzgFileTestFactory;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcRueckfrage;

@ExtendWith(MockitoExtension.class)
class NachrichtMapperTest {

	@Mock
	private FileService fileService;

	@InjectMocks
	private NachrichtMapper mapper;

	@Test
	void shouldMapEmptyAttachmentFields() {
		var rueckfrage = AntragraumproxyGrpcRueckfrageTestFactory.createRueckfrage();
		rueckfrage.setAttachmentFileId(null);
		var nachricht = mapper.fromRestRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

		assertThat(nachricht.attachments()).isEmpty();
	}

	@Nested
	class TestFromRestRueckfrage {

		private final AntragraumproxyGrpcRueckfrage rueckfrage = AntragraumproxyGrpcRueckfrageTestFactory.createRueckfrage();
		private final AntragraumproxyGrpcRueckfrage answeredRueckfrage = AntragraumproxyGrpcRueckfrageTestFactory.createRueckfrageWithReply();

		@BeforeEach
		void setup() {
			lenient().when(fileService.getFile(any(FileIdentificationData.class))).thenReturn(OzgFileTestFactory.create());
		}

		@Test
		void shouldMapNachrichtId() {
			var rueckfrage = AntragraumproxyGrpcRueckfrageTestFactory.createRueckfrage();
			rueckfrage.setId(NachrichtTestFactory.ID);
			var nachricht = mapper.fromRestRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.id()).isEqualTo(NachrichtTestFactory.ID);
		}

		@Test
		void shouldMapNachrichtEventId() {
			var rueckfrage = AntragraumproxyGrpcRueckfrageTestFactory.createRueckfrage();
			rueckfrage.setId(NachrichtTestFactory.ID);
			var nachricht = mapper.fromRestRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.nachrichtEventId()).isEqualTo(NachrichtTestFactory.NACHRICHT_EVENT_ID);
		}

		@Test
		void shouldMapVorgangId() {
			var nachricht = mapper.fromRestRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.vorgangId()).isEqualTo(AntragraumproxyGrpcRueckfrageTestFactory.VORGANG_ID);
		}

		@Test
		void shouldMapVorgangName() {
			var nachricht = mapper.fromRestRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.title()).isEqualTo(NachrichtTestFactory.VORGANG_TITLE);
		}

		@Test
		void shouldMapNachrichtText() {
			var nachricht = mapper.fromRestRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.message()).isEqualTo(AntragraumproxyGrpcRueckfrageTestFactory.TEXT);
		}

		@Test
		void shouldMapAnsweredAt() {
			var nachricht = mapper.fromRestRueckfrage(answeredRueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.answeredAt()).isEqualTo(AntragraumproxyGrpcRueckfrageTestFactory.ANSWERED_AT_DATE);
		}

		@Test
		void shouldMapSendAt() {
			var nachricht = mapper.fromRestRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.sendAt()).isEqualTo(AntragraumproxyGrpcRueckfrageTestFactory.SEND_AT_DATE);
		}

		@Test
		void shouldMapInvalidAnsweredAt() {
			var rueckfrage = AntragraumproxyGrpcRueckfrageTestFactory.createRueckfrage();
			rueckfrage.setAnsweredAt("huhu");
			var nachricht = mapper.fromRestRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.answeredAt()).isZero();
		}

		@Test
		void shouldMapEmptyAnsweredAt() {
			var rueckfrage = AntragraumproxyGrpcRueckfrageTestFactory.createRueckfrage();
			rueckfrage.setAnsweredAt("");
			var nachricht = mapper.fromRestRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.answeredAt()).isZero();
		}

		@Test
		void shouldMapAttachments() {
			var nachricht = mapper.fromRestRueckfrage(AntragraumproxyGrpcRueckfrageTestFactory.createRueckfrage(),
			  NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.attachments()).hasSize(2);
		}

		@Test
		void shouldMapEmptyAttachments() {
			when(fileService.getFile(any(FileIdentificationData.class))).thenReturn(null);

			var nachricht = mapper.fromRestRueckfrage(AntragraumproxyGrpcRueckfrageTestFactory.createRueckfrage(),
			  NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.attachments()).isEmpty();
		}

		@Test
		void shouldMapAccessible() {
			var nachricht = mapper.fromRestRueckfrage(answeredRueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.accessible()).isEqualTo(NachrichtTestFactory.ACCESSIBLE);
		}

		@Test
		void shouldMapTrustLevel() {
			var nachricht = mapper.fromRestRueckfrage(answeredRueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.trustLevel()).isEqualTo(NachrichtTestFactory.TRUST_LEVEL);
		}

		@Test
		void shouldNotHaveAnswersWhenUnanswered() {
			var nachricht = mapper.fromRestRueckfrage(AntragraumproxyGrpcRueckfrageTestFactory.createRueckfrage(),
			  NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.replyNachrichten()).isEmpty();
		}

		@Test
		void shouldMapAnswerNachrichtId() {
			var nachricht = mapper.fromRestRueckfrage(answeredRueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.replyNachrichten().getFirst().id()).isEqualTo(AntragraumproxyGrpcRueckfrageTestFactory.ANSWER_ID);
		}

		@Test
		void shouldMapAnswerRueckfrageId() {
			var nachricht = mapper.fromRestRueckfrage(answeredRueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.replyNachrichten().getFirst().rueckfrageId()).isEqualTo(AntragraumproxyGrpcRueckfrageTestFactory.RUECKFRAGE_ID);
		}

		@Test
		void shouldMapAnswerNachrichtText() {
			var nachricht = mapper.fromRestRueckfrage(answeredRueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.replyNachrichten().getFirst().message()).isEqualTo(AntragraumproxyGrpcRueckfrageTestFactory.ANSWER_TEXT);
		}

		@Test
		void shouldMapAnswerAttachments() {
			var nachricht = mapper.fromRestRueckfrage(answeredRueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.replyNachrichten().getFirst().attachments()).hasSize(1);
		}

		@Test
		void shouldMapStatus() {
			var nachricht = mapper.fromRestRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.status()).isEqualTo(NachrichtTestFactory.STATUS);
		}

	}

	@Nested
	class TestToGrpcRueckfrageAnswer {

		@Test
		void shouldMapNachrichtText() {
			var nachricht = mapper.toRestRueckfrageAnswer(ReplyNachrichtTestFactory.create());

			assertThat(nachricht.getAnswerText()).isEqualTo(ReplyNachrichtTestFactory.TEXT);
		}

		@Test
		void shouldMapEmptyAttachments() {
			var nachricht = mapper.toRestRueckfrageAnswer(ReplyNachrichtTestFactory.create());

			assertThat(nachricht.getAttachmentFileId()).isEmpty();
		}

		@Test
		void shouldMapAttachments() {
			var nachricht = mapper.toRestRueckfrageAnswer(
			  ReplyNachrichtTestFactory.createBuilder()
				.attachments(List.of(OzgFile.builder().id(UUID.randomUUID().toString()).build())).build());

			assertThat(nachricht.getAttachmentFileId()).hasSize(1);
		}

		@Test
		void shouldNotMapEmptyAttachments() {
			var rueckfrage = AntragraumproxyGrpcRueckfrageTestFactory.createRueckfrage();
			rueckfrage.setAttachmentFileId(List.of());

			var nachricht = mapper.fromRestRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.attachments()).isEmpty();
		}

	}

}