/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static de.ozgcloud.antragsraum.nachricht.NachrichtenService.*;
import static org.mockito.Mockito.*;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.nio.charset.Charset;
import java.util.List;

import org.apache.commons.lang3.RandomStringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import com.fasterxml.jackson.databind.ObjectMapper;

import de.ozgcloud.antragsraum.attachments.FileService;
import de.ozgcloud.antragsraum.command.CommandReference;
import de.ozgcloud.antragsraum.command.CommandReferenceTestFactory;
import de.ozgcloud.antragsraum.common.AddressNotFoundException;
import de.ozgcloud.antragsraum.common.TechnicalException;

@AutoConfigureMockMvc
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.MOCK)
@SpringJUnitConfig(classes = { NachrichtenControllerTestConfiguration.class })
public class NachrichtenControllerITCase {

	@SpyBean
	private NachrichtenController nachrichtenController;

	@Autowired
	private MockMvc mockMvc;

	@MockBean
	private NachrichtenService nachrichtenService;

	@MockBean
	private FileService fileService;

	@Nested
	class TestLoadingRueckfrageHeaders {

		@Test
		@WithMockUser
		void shouldLoadRueckfragenHeader() throws Exception {
			when(nachrichtenService.getRueckfrageHeadersOfPostfach(NachrichtTestFactory.POSTFACH_ID)).thenReturn(
			  List.of(RueckfrageTestFactory.createRueckfrageHeader()));

			performRequest()
			  .andExpect(status().isOk())
			  .andExpect(jsonPath("$[0]['nachrichtHeader']['id']").value(NachrichtTestFactory.ID));
		}

		@Test
		void shouldNotLoadRueckfragenHeaderUnauthorized() throws Exception {
			performRequest().andExpect(status().isUnauthorized());
		}

		@Test
		@WithMockUser
		void shouldOnlyIncludeSelfLink() throws Exception {
			var headerWithSelfLink = RueckfrageTestFactory.createRueckfrageHeader();
			headerWithSelfLink.add(linkTo(NachrichtenController.class).withSelfRel());

			when(nachrichtenService.getRueckfrageHeadersOfPostfach(NachrichtTestFactory.POSTFACH_ID)).thenReturn(List.of(headerWithSelfLink));

			performRequest()
			  .andExpect(status().isOk())
			  .andExpect(jsonPath("$[0]['links'].length()").value(1))
			  .andExpect(jsonPath("$[0]['links'][0]['rel']").value("self"))
			  .andExpect(jsonPath("$[0]['links'][0]['href']").isString());
		}

		@Test
		@WithMockUser
		void shouldIncludeSelfAndRueckfrageLink() throws Exception {
			var fullyLinkedRueckfrageHeader = RueckfrageTestFactory.createRueckfrageHeader();
			fullyLinkedRueckfrageHeader.add(linkTo(NachrichtenController.class).withSelfRel());
			fullyLinkedRueckfrageHeader.add(linkTo(
			  methodOn(NachrichtenController.class).getRueckfrage(fullyLinkedRueckfrageHeader.getNachrichtHeader().id(),
				fullyLinkedRueckfrageHeader.getNachrichtHeader().nachrichtEventId())).withRel(RUECKFRAGE_LINK_RELATIONSHIP_NAME));

			when(nachrichtenService.getRueckfrageHeadersOfPostfach(NachrichtTestFactory.POSTFACH_ID)).thenReturn(
			  List.of(fullyLinkedRueckfrageHeader));

			performRequest()
			  .andExpect(status().isOk())
			  .andExpect(jsonPath("$[0]['links'].length()").value(2))
			  .andExpect(jsonPath("$[0]['links'][0]['rel']").value("self"))
			  .andExpect(jsonPath("$[0]['links'][0]['href']").isString())
			  .andExpect(jsonPath("$[0]['links'][1]['rel']").value(RUECKFRAGE_LINK_RELATIONSHIP_NAME))
			  .andExpect(jsonPath("$[0]['links'][1]['href']").isString());
		}

		@Test
		@WithMockUser
		void shouldHandleRuntimeExceptions() throws Exception {
			doThrow(RuntimeException.class).when(nachrichtenService).getRueckfrageHeadersOfPostfach(NachrichtTestFactory.POSTFACH_ID);

			performRequest().andExpect(status().isInternalServerError());
		}

		@Test
		@WithMockUser
		void shouldHandleTechnicalException() throws Exception {
			doThrow(TechnicalException.class).when(nachrichtenService).getRueckfrageHeadersOfPostfach(NachrichtTestFactory.POSTFACH_ID);

			performRequest().andExpect(status().isInternalServerError());
		}

		ResultActions performRequest() throws Exception {
			return mockMvc.perform(
			  get(NachrichtenController.PATH + "/rueckfragen/" + NachrichtTestFactory.POSTFACH_ID)
				.contentType(MediaType.APPLICATION_JSON).characterEncoding(Charset.defaultCharset()));
		}

	}

	@Nested
	class TestLoadingRueckfrage {

		@BeforeEach
		void init() {
			when(nachrichtenService.getRueckfrage(NachrichtTestFactory.ID, NachrichtTestFactory.NACHRICHT_EVENT_ID)).thenReturn(
			  RueckfrageTestFactory.createRueckfrage());
		}

		@Test
		@WithMockUser
		void shouldLoadRueckfrage() throws Exception {
			performRequest()
			  .andExpect(status().isOk())
			  .andExpect(jsonPath("$['nachricht']['id']").value(NachrichtTestFactory.ID));
		}

		@Test
		void shouldNotLoadRueckfrageUnauthorized() throws Exception {
			performRequest().andExpect(status().isUnauthorized());
		}

		@Test
		@WithMockUser
		void shouldHandleRuntimeExceptions() throws Exception {
			doThrow(RuntimeException.class).when(nachrichtenService).getRueckfrage(NachrichtTestFactory.ID, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			performRequest().andExpect(status().isInternalServerError());
		}

		@Test
		@WithMockUser
		void shouldHandleTechnicalException() throws Exception {
			doThrow(TechnicalException.class).when(nachrichtenService)
			  .getRueckfrage(NachrichtTestFactory.ID, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			performRequest().andExpect(status().isInternalServerError());
		}

		ResultActions performRequest() throws Exception {
			return mockMvc.perform(
			  get(NachrichtenController.PATH + "/rueckfrage/" + NachrichtTestFactory.ID + "/" + NachrichtTestFactory.NACHRICHT_EVENT_ID)
				.contentType(MediaType.APPLICATION_JSON).characterEncoding(Charset.defaultCharset()));
		}

	}

	@Nested
	class TestSendingNachricht {

		private final ObjectMapper mapper = new ObjectMapper();
		private final ReplyNachricht msg = ReplyNachrichtTestFactory.create();
		private final CommandReference commandReference = CommandReferenceTestFactory.create();

		@BeforeEach
		void init() {
			doReturn(commandReference).when(nachrichtenService).sendRueckfrageAnswer(any(ReplyNachricht.class));
		}

		@Test
		@WithMockUser
		void shouldSendNachricht() throws Exception {
			performPutRequest(mapper.writeValueAsString(msg))
			  .andExpect(status().isOk())
			  .andExpect(jsonPath("$.id").value(CommandReferenceTestFactory.ID))
			  .andExpect(jsonPath("$.nachrichtEventId").value(CommandReferenceTestFactory.NACHRICHT_EVENT_ID));
		}

		@Test
		@WithMockUser
		void shouldNotSendNachrichtWithoutAddress() throws Exception {
			doThrow(AddressNotFoundException.class).when(nachrichtenService).sendRueckfrageAnswer(any());

			performPutRequest(mapper.writeValueAsString(msg)).andExpect(status().isNotFound());
		}

		@Test
		@WithMockUser
		void shouldHandleRuntimeExceptions() throws Exception {
			doThrow(RuntimeException.class).when(nachrichtenService).sendRueckfrageAnswer(any());

			performPutRequest(mapper.writeValueAsString(msg)).andExpect(status().isInternalServerError());
		}

		@Nested
		class TestSendingInvalidReply {

			@Test
			@WithMockUser
			void shouldNotSendNachrichtBlankRueckfrageId() throws Exception {
				var invalidMsg = ReplyNachrichtTestFactory.createBuilder().rueckfrageId("").build();

				performPutRequest(mapper.writeValueAsString(invalidMsg)).andExpect(status().isBadRequest());
			}

			@Test
			@WithMockUser
			void shouldHandleInvalidRueckfrageId() throws Exception {
				var invalidMsg = msg.toBuilder().rueckfrageId("id_that_is_too_long_123456789").build();

				performPutRequest(mapper.writeValueAsString(invalidMsg)).andExpect(status().isBadRequest());
			}

			@Test
			@WithMockUser
			void shouldHandleInvalidNachrichtEventId() throws Exception {
				var invalidMsg = msg.toBuilder().nachrichtEventId("id_that_is_too_long_123456789").build();

				performPutRequest(mapper.writeValueAsString(invalidMsg)).andExpect(status().isBadRequest());
			}

			@Test
			@WithMockUser
			void shouldHandleInvalidPostfachId() throws Exception {
				var invalidMsg = msg.toBuilder().id("postfach_id_that_is_too_long_123456789_123456789").build();

				performPutRequest(mapper.writeValueAsString(invalidMsg)).andExpect(status().isBadRequest());
			}

			@Test
			@WithMockUser
			void shouldHandleInvalidMessage() throws Exception {
				var invalidMsg = msg.toBuilder().message(RandomStringUtils.random(500001, true, false)).build();

				performPutRequest(mapper.writeValueAsString(invalidMsg)).andExpect(status().isBadRequest());
			}

		}

		ResultActions performPutRequest(String body) throws Exception {
			return mockMvc.perform(
			  put(NachrichtenController.PATH + "/antwort")
				.with(csrf().asHeader())
				.contentType(MediaType.APPLICATION_JSON)
				.characterEncoding(Charset.defaultCharset())
				.content(body));
		}

	}

}