/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.apache.commons.lang3.RandomStringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;

import de.ozgcloud.antragsraum.attachments.OzgFile;
import de.ozgcloud.antragsraum.attachments.OzgFileTestFactory;
import de.ozgcloud.antragsraum.command.CommandReferenceTestFactory;
import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.antragsraum.events.NachrichtEventService;
import de.ozgcloud.antragsraum.events.NachrichtEventTestFactory;

@SpringBootTest
@SpringJUnitConfig(classes = { NachrichtServiceTestConfiguration.class })
class NachrichtenServiceITCase {

	public static final int OVERSIZED_EVENT_ID = 25;
	public static final int OVERSIZED_ID = 25;
	public static final int OVERSIZED_POSTFACH_ID = UUID.randomUUID().toString().length() + 1;
	public static final int OVERSIZED_TEXT = 100_001;
	public static final int MAX_ATTACHMENT_COUNT = 21;
	@Autowired
	private NachrichtenService nachrichtenService;
	@MockBean
	private NachrichtEventService nachrichtEventService;
	@MockBean
	private NachrichtenRemoteService nachrichtenRemoteService;
	@MockBean
	private NachrichtMapper nachrichtMapper;
	@MockBean
	private NachrichtenController nachrichtenController;

	@Nested
	class TestSendNachricht {

		@BeforeEach
		void init() {
			when(nachrichtEventService.getNachrichtEventById(anyString())).thenReturn(NachrichtEventTestFactory.create());
			when(nachrichtenRemoteService.sendAnswer(any(ReplyNachricht.class), anyString())).thenReturn(CommandReferenceTestFactory.create());
		}

		@Test
		void shouldSendValidReply() {
			ReplyNachricht msg = ReplyNachrichtTestFactory.create();

			nachrichtenService.sendRueckfrageAnswer(msg);

			verify(nachrichtenRemoteService).sendAnswer(any(ReplyNachricht.class), anyString());
		}

		@Nested
		class TestSendingInvalidReply {

			@Test
			void messageTestTooLong() {
				ReplyNachricht msg = ReplyNachrichtTestFactory.createBuilder().message(RandomStringUtils.random(OVERSIZED_TEXT)).build();

				assertThatExceptionOfType(TechnicalException.class).isThrownBy(() -> nachrichtenService.sendRueckfrageAnswer(msg))
				  .withMessageStartingWith("TechnicalException happened! Message: message: Message text too long");
			}

			@Test
			void messagePostfachIdTooLong() {
				ReplyNachricht msg = ReplyNachrichtTestFactory.createBuilder().postfachId(RandomStringUtils.random(OVERSIZED_POSTFACH_ID)).build();

				assertThatExceptionOfType(TechnicalException.class).isThrownBy(() -> nachrichtenService.sendRueckfrageAnswer(msg))
				  .withMessageStartingWith("TechnicalException happened! Message: postfachId: An id is too long");
			}

			@Test
			void messageIdTooLong() {
				ReplyNachricht msg = ReplyNachrichtTestFactory.createBuilder().id(RandomStringUtils.random(OVERSIZED_ID)).build();

				assertThatExceptionOfType(TechnicalException.class).isThrownBy(() -> nachrichtenService.sendRueckfrageAnswer(msg))
				  .withMessageStartingWith("TechnicalException happened! Message: id: An id is too long");
			}

			@Test
			void messageNachrichtEventIdTooLong() {
				ReplyNachricht msg = ReplyNachrichtTestFactory.createBuilder().nachrichtEventId(RandomStringUtils.random(OVERSIZED_EVENT_ID)).build();

				assertThatExceptionOfType(TechnicalException.class).isThrownBy(() -> nachrichtenService.sendRueckfrageAnswer(msg))
				  .withMessageStartingWith("TechnicalException happened! Message: nachrichtEventId: An id id too long");
			}

			@Test
			void messageTooManyAttachments() {
				List<OzgFile> attachments = new ArrayList<>(MAX_ATTACHMENT_COUNT);
				for (int i = 0; i < MAX_ATTACHMENT_COUNT; i++) {
					attachments.add(OzgFileTestFactory.create());
				}

				ReplyNachricht msg = ReplyNachrichtTestFactory.createBuilder().attachments(attachments).build();

				assertThatExceptionOfType(TechnicalException.class).isThrownBy(() -> nachrichtenService.sendRueckfrageAnswer(msg))
				  .withMessageStartingWith("TechnicalException happened! Message: attachments: size must be between 0 and 20");
			}

		}

	}

}