/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.security;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.io.IOException;
import java.util.Optional;

import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.RandomStringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.test.util.ReflectionTestUtils;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jws;
import io.jsonwebtoken.JwtParser;

@ExtendWith(MockitoExtension.class)
class JwtTokenFilterTest {

	@Mock
	HttpServletRequest request;
	@Mock
	HttpServletResponse response;
	@Mock
	FilterChain chain;
	private static final String SECRET = RandomStringUtils.random(256);
	@Mock
	private JwtTokenVerifier tokenVerifier;
	@Mock
	private InMemoryUserDetailService userDetailService;
	@Spy
	@InjectMocks
	private JwtTokenFilter tokenFilter;
	private String token;

	@Nested
	class TestTokenFilterProcessing {

		@BeforeEach
		void setup() {
			User user = UserTestFactory.create();

			JwtTokenProvider tokenProvider = new JwtTokenProvider();
			ReflectionTestUtils.setField(tokenProvider, "jwtSecret", SECRET);

			token = tokenProvider.generate(user);
		}

		@Test
		void shouldCallChainDoFilter() throws ServletException, IOException {
			tokenFilter.doFilterInternal(request, response, chain);

			verify(chain).doFilter(any(HttpServletRequest.class), any(HttpServletResponse.class));
		}

		@Test
		void shouldThrowException() {
			when(request.getHeader(anyString())).thenReturn(JwtTokenFilter.TOKEN_PREFIX + " error_token");
			doThrow(IllegalStateException.class).when(tokenVerifier).validate(anyString());

			assertThatExceptionOfType(AccessDeniedException.class).isThrownBy(() ->
			  tokenFilter.doFilterInternal(request, response, chain)).withMessage("User not found or invalid token");
		}

		@Test
		void shouldCallGetJwtFromRequest() throws ServletException, IOException {
			tokenFilter.doFilterInternal(request, response, chain);

			verify(tokenFilter).getJwtFromRequest(any(HttpServletRequest.class));
		}

		@Test
		void shouldCallTokenVerifierValidate() throws ServletException, IOException {
			when(request.getHeader(JwtTokenFilter.TOKEN_HEADER)).thenReturn(JwtTokenFilter.TOKEN_PREFIX + " " + token);
			when(tokenFilter.getJwtFromRequest(request)).thenReturn(Optional.of(token));

			tokenFilter.doFilterInternal(request, response, chain);

			verify(tokenVerifier).validate(any());
		}

		@Test
		void shouldCallTokenVerifierGetJws() throws ServletException, IOException {
			when(request.getHeader(JwtTokenFilter.TOKEN_HEADER)).thenReturn(JwtTokenFilter.TOKEN_PREFIX + " " + token);
			when(tokenFilter.getJwtFromRequest(request)).thenReturn(Optional.of(token));
			when(tokenVerifier.validate(any())).thenReturn(Optional.of(mock(JwtParser.class)));

			tokenFilter.doFilterInternal(request, response, chain);

			verify(tokenVerifier).getJws(any(), any());
		}

		@Test
		void shouldNotCallTokenVerifierGetJws() throws ServletException, IOException {
			when(request.getHeader(JwtTokenFilter.TOKEN_HEADER)).thenReturn(JwtTokenFilter.TOKEN_PREFIX + " " + token);
			when(tokenFilter.getJwtFromRequest(request)).thenReturn(Optional.of(token));
			when(tokenVerifier.validate(any())).thenReturn(Optional.empty());

			tokenFilter.doFilterInternal(request, response, chain);

			verify(tokenVerifier, never()).getJws(any(), any());
		}

		@Test
		void shouldCallCreateAuthentication() throws ServletException, IOException {
			initFilterFlow();

			tokenFilter.doFilterInternal(request, response, chain);

			verify(tokenFilter).createAuthentication(any(HttpServletRequest.class), anyString(), any());
		}

		@Test
		void shouldCallSetUserToSecurityContext() throws ServletException, IOException {
			initFilterFlow();

			tokenFilter.doFilterInternal(request, response, chain);

			verify(tokenFilter).setUserToSecurityContext(any(Authentication.class));
		}

		private void initFilterFlow() {
			when(request.getHeader(JwtTokenFilter.TOKEN_HEADER)).thenReturn(JwtTokenFilter.TOKEN_PREFIX + " " + token);
			when(tokenFilter.getJwtFromRequest(request)).thenReturn(Optional.of(token));
			when(tokenVerifier.validate(any())).thenReturn(Optional.of(mock(JwtParser.class)));
			Jws<Claims> jws = mock(Jws.class);
			Claims claims = mock(Claims.class);
			when(claims.getSubject()).thenReturn("username");
			when(jws.getPayload()).thenReturn(claims);
			when(tokenVerifier.getJws(any(), any())).thenReturn(Optional.of(jws));
			when(userDetailService.loadUserByUsername(any())).thenReturn(UserTestFactory.create());
		}

	}

	@Nested
	class TestSettingSecurityContext {

		@BeforeEach
		void setup() {
			User user = UserTestFactory.create();

			JwtTokenProvider tokenProvider = new JwtTokenProvider();
			ReflectionTestUtils.setField(tokenProvider, "jwtSecret", SECRET);

			token = tokenProvider.generate(user);

			when(userDetailService.loadUserByUsername(any())).thenReturn(UserTestFactory.create());
		}

		@Test
		void shouldCreateAuthentication() {
			var auth = tokenFilter.createAuthentication(request, UserTestFactory.USER_NAME, Optional.of(token));

			assertThat(auth).isNotNull();
		}

		@Test
		void shouldBeAuthenticated() {
			var auth = tokenFilter.createAuthentication(request, UserTestFactory.USER_NAME, Optional.of(token));

			assertThat(auth.isAuthenticated()).isTrue();
		}

		@Test
		void shouldBeUsernamePasswordAuthenticationToken() {
			var auth = tokenFilter.createAuthentication(request, UserTestFactory.USER_NAME, Optional.of(token));

			assertThat(auth).isInstanceOf(UsernamePasswordAuthenticationToken.class);
		}

		@Test
		void shouldHaveAuthorities() {
			var auth = tokenFilter.createAuthentication(request, UserTestFactory.USER_NAME, Optional.of(token));

			assertThat(((UsernamePasswordAuthenticationToken) auth).getAuthorities()).isNotNull();
		}

		@Test
		void shouldHandleEmptyToken() {
			var auth = tokenFilter.createAuthentication(request, UserTestFactory.USER_NAME, Optional.empty());

			assertThat(((UsernamePasswordAuthenticationToken) auth).getAuthorities()).isNotNull();
		}

	}

	@Nested
	class TestLoadingTokenFromRequest {

		@BeforeEach
		void setup() {
			User user = UserTestFactory.create();

			JwtTokenProvider tokenProvider = new JwtTokenProvider();
			ReflectionTestUtils.setField(tokenProvider, "jwtSecret", SECRET);

			token = tokenProvider.generate(user);

			when(request.getHeader(JwtTokenFilter.TOKEN_HEADER)).thenReturn(JwtTokenFilter.TOKEN_PREFIX + token);
		}

		@Test
		void shouldGetToken() {
			var tokenOptional = tokenFilter.getJwtFromRequest(request);

			assertThat(tokenOptional).isPresent();
		}

		@Test
		void shouldNotGetToken() {
			when(request.getHeader(JwtTokenFilter.TOKEN_HEADER)).thenReturn("other " + token);

			var tokenOptional = tokenFilter.getJwtFromRequest(request);

			assertThat(tokenOptional).isNotPresent();
		}

		@Test
		void shouldGetTokenValue() {
			var tokenOptional = tokenFilter.getJwtFromRequest(request);

			assertThat(tokenOptional).isPresent().map(tokenValue -> assertThat(tokenValue).isEqualTo(token));
		}

	}

	@Nested
	class TestNullHandling {

		@Test
		void shouldThrowIllegalStateException() {
			assertThatIllegalArgumentException().isThrownBy(() -> tokenFilter.createAuthentication(null, null, null));
		}

		@Test
		void shouldThrowIllegalArgumentException() {
			assertThatIllegalArgumentException().isThrownBy(() -> tokenFilter.doFilterInternal(null, null, null));
		}

	}

}